<?php

/**
 * @file
 * Hooks and preprocess functions for the Mason module.
 */

use Drupal\Core\Template\Attribute;
use Drupal\mason\Entity\Mason;

/**
 * Prepares variables for mason.html.twig templates.
 */
function template_preprocess_mason(&$variables) {
  $element  = $variables['element'];
  $defaults = Mason::htmlSettings();
  $settings = isset($element['#settings']) ? array_merge($defaults, $element['#settings']) : $defaults;
  $json     = isset($element['#json']) ? $element['#json'] : '';

  // Prepare attributes, and add the configuration as JSON object.
  $id = Mason::getHtmlId('mason', $settings['id']);
  $attributes = new Attribute();
  $attributes->setAttribute('id', $id);
  $attributes->setAttribute('data-mason', $json);

  // Pass settings and attributes to twig.
  $settings['class_by_id'] = strpos($id, 'mason-') !== FALSE ? str_replace('mason-', 'mason--', $id) : '';
  $variables['settings']   = $settings;
  $variables['attributes'] = $attributes;

  // Process individual item.
  $variables['items'] = [];
  foreach ($element['#items'] as $delta => $item) {
    $settings = isset($item['settings']) ? array_merge($settings, $item['settings']) : $settings;
    unset($item['settings']);

    $box = [
      '#theme'    => 'mason_box',
      '#item'     => $item,
      '#settings' => $settings,
      '#delta'    => $delta,
    ];
    $variables['items'][$delta] = $box;
    unset($box);
  }
}

/**
 * Prepares variables for mason-box.html.twig templates.
 */
function template_preprocess_mason_box(&$variables) {
  foreach (['attributes', 'delta', 'item', 'settings'] as $key) {
    $variables[$key] = isset($variables['element']["#$key"]) ? $variables['element']["#$key"] : [];
  }

  // Boxes may have captions.
  foreach (['data', 'link', 'title'] as $key) {
    $variables['item']['caption'][$key] = isset($variables['item']['caption'][$key]) ? $variables['item']['caption'][$key] : [];
  }

  $item     = &$variables['item'];
  $image    = $item['image'];
  $settings = &$variables['settings'];

  $attributes = new Attribute();
  if ($settings['background'] && !empty($image['image_url'])) {

    if (!empty($settings['lazyload'])) {
      $attributes->addClass($settings['lazyload']);
      $attributes->setAttribute('data-src', $image['image_url']);
    }
    else {
      $attributes->setAttribute('style', 'background-image: url(' . $image['image_url'] . ')');
    }
  }

  $item['caption'] = array_filter($item['caption']);

  $variables['attributes'] = $attributes;
  $variables['fillers']    = !empty($settings['fillers']) && ($variables['delta'] > $settings['fillers']);
}
